//#######################################################################
//#
//#	Copyright 	Earth People Technology Inc. 2013
//#
//#
//# File Name:  EPT_4CE6_AF_D1_Top.v
//# Author:     Earth People Technology
//# Date:       November 26, 2014
//# Revision:   A
//#
//# Development: EPT I2C Slave Project 
//# Application: Altera Cyclone IV FPGA
//# Description: This file contains verilog code which will allow access
//#              to Active Transfer Library. 
//#               
//#              
//#
//#************************************************************************
//#
//# Revision History:	
//#			DATE		VERSION		DETAILS		
//#			11/26/14 	1			Created
//#
//#									
//#
//#######################################################################


//************************************************************************
//* Module Declaration
//************************************************************************

module EPT_4CE6_AF_D1_Top (

	
	input  wire [1:0]           aa,
	input  wire [1:0]           bc_in,
	output wire [2:0]           bc_out,
	inout  wire [7:0]		    bd_inout,
	
	input wire   [7:0]          LB_IO1,       //XIOH -- J10
	input wire   [7:0]          LB_IO2,       //AD   -- J9
	input wire   [7:0]          LB_IO3,       //XIOH -- J10
	output wire  [7:0]          LB_IO4,       //XIOL -- J8
	
	inout wire                  I2C_SDA,
	input wire                  I2C_SCL,
	
	//TEST ONLY!
	output wire  [11:0]          TEST_BUS_1,
    output wire  [3:0]          STATE_OUT,
	
	
	//Switch and LED Control Signals
	input wire                SW_USER_1,
	input wire                SW_USER_2,

	output wire  [3:0]         LED
	);

   //-----------------------------------------------
   // Parameters
   //-----------------------------------------------
   

   parameter                   GLOBAL_RESET_COUNT = 12'h9c8;
   
 
//***************************************************************************
//* Internal Signals and Registers Declarations
//***************************************************************************
    wire                       CLK_66;
	wire                       RST;

	wire [23:0]                UC_IN;
	wire [21:0]                UC_OUT;
	
	//Trigger Signals
	wire  [7:0]                trigger_out;
	wire [7:0]                 trigger_in_byte;	
	
	//LED registers
	wire                       led_reset;
	
	//Transfer registers
	reg                        start_transfer;
	reg                        start_transfer_reg;
	reg   [7:0]                transfer_out_byte;

    //Transfer Control registers
	wire                       transfer_in_received;
	wire [7:0]                 transfer_in_byte;
	reg  [7:0]                 transfer_control_byte;
	wire   [7:0]               control_register;
	
	//Reset signals
	wire                       reset;
	reg [11:0]                 reset_counter;
	reg                        reset_signal_reg;
	
	//Serial I2C Receive Registers
	reg                        serial_receive_start_transfer;
	reg                        serial_receive_start_transfer_reg;
	wire                       serial_receive_in_received;
	wire  [7:0]                serial_receive_out_byte;
	wire  [7:0]                serial_control_in_byte;	
	wire                       i2c_receive_word_ready;

	//Serial I2C Transmit Registers
	reg                        serial_transmit_start_transfer;
	wire                       serial_transmit_in_received;
	reg  [7:0]                 serial_transmit_out_byte;
	wire [7:0]                 serial_transmit_in_byte;	
	wire                       i2c_transmit_word_enable;
	
	//TEST ONLY
	wire  [11:0]               i2c_slave_comms_test_bus_out;
	wire  [3:0]               i2c_slave_comms_state;
	
	
`ifdef SIM
   reg [8*26:1] state_name;
`endif
	


//***************************************************************************
//* 	Signal Assignments	
//***************************************************************************

   //Clock and Reset
   assign            CLK_66 = aa[1];
   assign            RST = reset;
   assign            reset = reset_signal_reg;
   
   //Set the trigger out to zero
   assign            trigger_in_byte = 8'h0;
   
   //Control Register Map
   assign            led_reset = control_register[2];
   
   assign            LED[0] = ~I2C_SCL;
   assign            LED[1] = 1'b0;
   assign            LED[2] = 1'b0;
   assign            LED[3] = 1'b0;
   
   assign            TEST_BUS_1[0] = i2c_slave_comms_test_bus_out[0];
   assign            TEST_BUS_1[1] = i2c_slave_comms_test_bus_out[1];
   assign            TEST_BUS_1[2] = i2c_slave_comms_test_bus_out[2];
   assign            TEST_BUS_1[3] = i2c_slave_comms_test_bus_out[3];
   assign            TEST_BUS_1[4] = i2c_slave_comms_test_bus_out[4];
   assign            TEST_BUS_1[5] = i2c_slave_comms_test_bus_out[5];
   assign            TEST_BUS_1[6] = i2c_slave_comms_test_bus_out[6];
   assign            TEST_BUS_1[7] = i2c_slave_comms_test_bus_out[7];
   assign            TEST_BUS_1[8] = i2c_slave_comms_test_bus_out[8];
   assign            TEST_BUS_1[9] = i2c_slave_comms_test_bus_out[9];
   assign            TEST_BUS_1[10] = i2c_slave_comms_test_bus_out[10];
   assign            TEST_BUS_1[11] = i2c_slave_comms_test_bus_out[11];
   
   assign            STATE_OUT = i2c_slave_comms_state;
   
   

//Trigger In Byte
   assign            trigger_out  = {6'h00,i2c_receive_word_ready,i2c_transmit_word_enable};

   
//**************************************************
//* 	Reset Signal	
//**************************************************
	always @(posedge CLK_66 or posedge aa[0])
	begin
	  if(aa[0])
	  begin
			reset_signal_reg <= 1'b0;
			reset_counter <= 0;
	  end
	  else
	  begin
		if( reset_counter < GLOBAL_RESET_COUNT )
		begin
			reset_signal_reg <= 1'b0;
			reset_counter <= reset_counter + 1'b1;
		end
		else
		begin
			reset_signal_reg <= 1'b1;
		end
	  end
	end

   //-----------------------------------------------
   // I2C Slave Unit Send received word to host  
   //-----------------------------------------------
	always @(posedge CLK_66 or negedge RST)
	begin
	  if(!RST)
	  begin
			serial_receive_start_transfer <= 1'b0;
			serial_receive_start_transfer_reg <= 1'b0;
	  end
	  else
	  begin
	        if(control_register[1] & !serial_receive_start_transfer_reg)
			begin
	           serial_receive_start_transfer <= 1'b1;
			   serial_receive_start_transfer_reg <= 1'b1;
			end
			else if(control_register[1] & serial_receive_start_transfer_reg)
			begin
	           serial_receive_start_transfer <= 1'b0;
			   serial_receive_start_transfer_reg <= 1'b1;
			end
			else if(!control_register[1] & serial_receive_start_transfer_reg)
			begin
	           serial_receive_start_transfer <= 1'b0;
			   serial_receive_start_transfer_reg <= 1'b0;
			end
	  end
	end
	
   //-----------------------------------------------
   // I2C Slave SERDES Module 
   //-----------------------------------------------
    i2c_slave_comms             I2C_SERDES_MODULE_INST
	(
	.CLK                        (CLK_66),
	.RESET                      (RST),

	.IIC_DATA                   (I2C_SDA),
	.IIC_CLOCK                  (I2C_SCL),

	.SERIAL_IIC_TRANSMIT_DATA_IN (serial_transmit_in_byte),
	.SERIAL_IIC_TRANSMIT_ENA_OUT (i2c_transmit_word_enable),
	.SERIAL_IIC_TRANSMIT_RDY_IN  (control_register[0]),

	.SERIAL_IIC_RECEIVE_DATA_OUT (serial_receive_out_byte),
	.SERIAL_IIC_RECEIVE_ENA_IN   (control_register[1]),
	.SERIAL_IIC_RECEIVE_RDY_OUT  (i2c_receive_word_ready),
	
	
	.ERROR_OUT                   (),
	
	.TEST_BUS_OUT                (i2c_slave_comms_test_bus_out),
	.STATE_OUT                   (i2c_slave_comms_state)
     );

   //-----------------------------------------------
   // Control Register from Transfer In 
   //-----------------------------------------------
  active_control_register      ACTIVE_CONTROL_REG_INST
  (
   .CLK                        (CLK_66),
   .RST                        (RST),
   .TRANSFER_IN_RECEIVED       (transfer_in_received),
   .TRANSFER_IN_BYTE           (transfer_in_byte),

   .CONTROL_REGISTER           (control_register)
   );
   

   //-----------------------------------------------
   // Instantiate the EPT Active Transfer Library
   //-----------------------------------------------

	active_transfer_library	   ACTIVE_TRANSFER_LIBRARY_INST
	(	
	.aa                        (aa),
	.bc_in                     (bc_in),
	.bc_out                    (bc_out),
	.bd_inout                  (bd_inout),

	.UC_IN                     (UC_IN),
	.UC_OUT                    (UC_OUT)
	
	);
	
   //-----------------------------------------------
   // Instantiate the EPT Active Modules
   //-----------------------------------------------
wire [22*4-1:0]  uc_out_m;
eptWireOR # (.N(4)) wireOR (UC_OUT, uc_out_m);
	active_trigger             ACTIVE_TRIGGER_INST
	(
	 .uc_clk                   (CLK_66),
	 .uc_reset                 (RST),
	 .uc_in                    (UC_IN),
	 .uc_out                   (uc_out_m[ 0*22 +: 22 ]),

	 .trigger_to_host          (trigger_out),
	 .trigger_to_device        (trigger_in_byte)
	
	);

	active_transfer            ACTIVE_TRANSFER_INST_1
	(
	 .uc_clk                   (CLK_66),
	 .uc_reset                 (RST),
	 .uc_in                    (UC_IN),
	 .uc_out                   (uc_out_m[ 1*22 +: 22 ]),
	
	 .start_transfer           (start_transfer),
	 .transfer_received        (transfer_in_received),
	 
	 .transfer_busy            (),
	
	 .uc_addr                  (3'h2),

	 .transfer_to_host         (transfer_out_byte),
	 .transfer_to_device       (transfer_in_byte)	
	);
	
	active_transfer            ACTIVE_TRANSFER_INST_2
	(
	 .uc_clk                   (CLK_66),
	 .uc_reset                 (RST),
	 .uc_in                    (UC_IN),
	 .uc_out                   (uc_out_m[ 2*22 +: 22 ]),
	
	 .start_transfer           (serial_transmit_start_transfer),
	 .transfer_received        (serial_transmit_in_received),
	 
	 .transfer_busy            (),
	
	 .uc_addr                  (3'h4),

	 .transfer_to_host         (serial_transmit_out_byte),
	 .transfer_to_device       (serial_transmit_in_byte)	
	);

	active_transfer            ACTIVE_TRANSFER_INST_3
	(
	 .uc_clk                   (CLK_66),
	 .uc_reset                 (RST),
	 .uc_in                    (UC_IN),
	 .uc_out                   (uc_out_m[ 3*22 +: 22 ]),
	
	 .start_transfer           (serial_receive_start_transfer),
	 .transfer_received        (serial_receive_in_received),
	 
	 .transfer_busy            (),
	
	 .uc_addr                  (3'h7),

	 .transfer_to_host         (serial_receive_out_byte),
	 .transfer_to_device       ()	
	);

	
endmodule
